<?php declare(strict_types=1);

namespace Apexl\Tests\User\FeatureTests;

use apexl\Io\TestInfrastructure\FeatureTest;

final class AuthenticationTest extends FeatureTest
{
    public function testNotAuthenticated(): void
    {
        $data = $this->get('/authenticated');
        $this->assertArrayHasKey('authenticated', $data);
        $this->assertFalse($data['authenticated']);
    }

    public function testAuthenticated(): void
    {
        $data = $this->getAuthenticated('/authenticated');
        $this->assertArrayHasKey('authenticated', $data);
        $this->assertTrue($data['authenticated']);
    }
}
