<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\collections;

use apexl\Io\modules\user\entities\userMetaEntity;
use Arrayy\Collection\AbstractCollection;

class userMetaCollection extends AbstractCollection
{

    public function getType(): string
    {
        return userMetaEntity::class;
    }
}
