<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\traits;

use app\vendor\apexl\utils\src\Utils\Strings\Append;

trait Appends
{
    public final const int FLAG_URL = 1;

    protected function append(string $base, string $append = '', $flags = self::FLAG_URL): string
    {
        return Append::append($base, $append, $flags);
    }

}