<?php

namespace apexl\Io\modules\user\entities\operators;

use apexl\Io\exceptions\RecordNotFoundException;
use apexl\Io\operators\entityDatabaseOperator;

class sessionOperator extends entityDatabaseOperator
{
    /**
     * @throws RecordNotFoundException
     */
    public function loadByUserId(int $userId): array
    {
        return $this->loadBy('uid', $userId);
    }
}
